#include <stdbool.h>
#include <string.h>
#include <stdint.h>
#include <stdio.h>
#include "printf.h"


//simple printf implementation. (c) 2019 Dmitry Grinberg. http://dmitry.gr   No commercial use whatsoever

typedef struct {
	
	void* dest;
	unsigned long maxChars;
	
	void* furtherCallback;
	void* furtherUserData;
	
} PrintData;

typedef char (*StrPrintfExCbk)(void* userData, char chr);	//return 0 to stop printing

static void StrPrvPrintfEx_number(uint64_t number, unsigned base, bool zeroExtend, bool isSigned, unsigned padToLength, bool caps){
	
	char buf[64];
	unsigned idx = sizeof(buf) - 1;
	unsigned chr, i;
	char neg = 0;
	unsigned numPrinted = 0;
	
	
	if(padToLength > 63) padToLength = 63;
	
	buf[idx--] = 0;	//terminate
	
	if(isSigned){
		
		if(number >> 63){
			
			neg = 1;
			number = -number;
		}
	}
	
	do{
		if(base == 16){
			chr = number & 0x0F;
			number >>= 4;
		}
		else if(base == 2){
			chr = number & 1;
			number >>= 1;
		}
		else{
			chr = number % base;
			number = number / base;
		}
		
		buf[idx--] = (chr >= 10) ? (chr + (caps ? 'A' : 'a') - 10) : (chr + '0');
		
		numPrinted++;
		
	}while(number);
	
	if(neg){
		
		buf[idx--] = '-';
		numPrinted++;
	}
	
	if(padToLength > numPrinted){
		
		padToLength -= numPrinted;
	}
	else{
		
		padToLength = 0;
	}
	
	while(padToLength--){
		
		buf[idx--] = zeroExtend ? '0' : ' ';
		numPrinted++;
	}
	
	idx++;
	
	for(i = 0; i < numPrinted; i++)
		prPutchar((buf + idx)[i]);
}

static unsigned StrVPrintf_StrLen_withMax(const char* s,unsigned max){
	
	unsigned len = 0;
	
	while((*s++) && (len < max)) len++;
	
	return len;
}

static unsigned StrVPrintf_StrLen(const char* s){
	
	unsigned len = 0;
	
	while(*s++) len++;
	
	return len;
}

void pr(const char* fmtStr, ...){
	
	unsigned long val64;
	char c;
	va_list vl;
	
	va_start(vl, fmtStr);
	
	
	while((c = *fmtStr++) != 0){
		
		if(c == '%'){
			
			bool zeroExtend = false, useLong = false, useLongLong = false, caps = false;
			unsigned padToLength = 0, len, i;
			const char* str;
			
more_fmt:
			
			c = *fmtStr++;
			
			switch(c){
				
				case '%':
					
					prPutchar(c);
					break;
				
				case 'c':
					
					prPutchar(va_arg(vl, unsigned int));
					break;
				
				case 's':
					
					str = va_arg(vl, char*);
					if(!str)
						str = "(null)";
					
					if(padToLength)
						len = StrVPrintf_StrLen_withMax(str, padToLength);
					else
						padToLength = len = StrVPrintf_StrLen(str);

					if(len > padToLength)
						len = padToLength;
					else {
						
						for(i = len; i < padToLength; i++)
							prPutchar(L' ');
					}
					for(i = 0; i < len; i++)
						prPutchar(*str++);
					
					break;
				
				case '0':
				case '.':
					
					if(!zeroExtend && !padToLength) {
						
						zeroExtend = true;
						goto more_fmt;
					}
				
				case '1':
				case '2':
				case '3':
				case '4':
				case '5':
				case '6':
				case '7':
				case '8':
				case '9':
					
					padToLength = (padToLength * 10) + c - '0';
					goto more_fmt;
				
				case 'u':
					
					val64 = useLongLong ? va_arg(vl, unsigned long long) : (useLong ? va_arg(vl,unsigned long) : va_arg(vl,unsigned));
					StrPrvPrintfEx_number(val64, 10, zeroExtend, false, padToLength, false);
					break;
					
				case 'd':
				case 'i':
					val64 = useLongLong ? va_arg(vl, signed long long) : (signed long long)(useLong ? va_arg(vl,signed long) : va_arg(vl,signed));
					StrPrvPrintfEx_number(val64, 10, zeroExtend, true, padToLength, false);
					break;
					
				case 'X':
					caps = true;
					
				case 'x':
					
					val64 = useLongLong ? va_arg(vl, unsigned long long) : (useLong ? va_arg(vl,unsigned long) : va_arg(vl,unsigned));
					StrPrvPrintfEx_number(val64, 16, zeroExtend, false, padToLength, caps);
					break;
					
				case 'L':
				case 'l':
					if (useLong)
						useLongLong = true;
					else
						useLong = true;
					goto more_fmt;
				
				default:
					
					prPutchar(c);
					break;
			}
		}
		else
			prPutchar(c);
	}
	
	va_end(vl);
}
