#ifndef __PSOC40xx__
#define __PSOC40xx__

#ifdef __cplusplus
 extern "C" {
#endif 


typedef enum IRQn
{
/******  Cortex-M0 Processor Exceptions Numbers ***************************************************/
  Reset_IRQn                    = -15,    /*!< 1 Reset Vector, invoked on Power up and warm reset*/  
  NonMaskableInt_IRQn           = -14,    /*!< 2 Non Maskable Interrupt                           */
  HardFault_IRQn                = -13,    /*!< 3 Cortex-M0 Hard Fault Interrupt                   */
  SVCall_IRQn                   = -5,     /*!< 11 Cortex-M0 SV Call Interrupt                     */
  PendSV_IRQn                   = -2,     /*!< 14 Cortex-M0 Pend SV Interrupt                     */
  SysTick_IRQn                  = -1,     /*!< 15 Cortex-M0 System Tick Interrupt                 */

/******  PSoC4 Specific Interrupt Numbers ********************************************************/

  GPIO0_IRQn                    =  0,
  GPIO1_IRQn                    =  1,
  GPIO2_IRQn                    =  2,
  GPIO3_IRQn                    =  3,
  WDT_IRQn                      =  4,
  SCB_IRQn                      =  5,
  SPC_IRQn                      =  6,
  CSD_IRQn                      =  7,
  TCPWM0_IRQn                   =  8,
} IRQn_Type;


#define __NVIC_PRIO_BITS          2         /*!< Number of Bits used for Priority Levels          */
#define __Vendor_SysTickConfig    0         /*!< Set to 1 if different SysTick Config is used     */

struct PsocGpioPrtType {
	struct {
		volatile uint32_t DR;
		volatile uint32_t PS;
		volatile uint32_t PC;
		volatile uint32_t INTR_CFG;
		volatile uint32_t INTR;
		uint32_t reserved1;
		volatile uint32_t PC2;
		uint32_t reserved2[9];
		volatile uint32_t DR_SET;
		volatile uint32_t DR_CLR;
		volatile uint32_t DR_INV;
		uint32_t reserved3[45];
	}PRT[4];
};

struct PsocHsiomPrtType {
	struct {
		volatile uint32_t SEL;
		uint32_t reserved3[63];
		
	}PRT[4];
};

struct PsocCpussType {
	volatile uint32_t CONFIG;
	volatile uint32_t SYSREQ;
	volatile uint32_t SYSARG;
	volatile uint32_t PROTECTION;	/*PRIVILEDGED*/
	volatile uint32_t PRIV_ROM;		/*PRIVILEDGED*/ //low16 - BROM, high16 - SROM
	volatile uint32_t PRIV_RAM;		/*PRIVILEDGED*/
	volatile uint32_t PRIV_FLASH;	/*PRIVILEDGED*/
	volatile uint32_t WOUNDING;		/*PRIVILEDGED*/
	uint32_t reserved3[4];
	volatile uint32_t FLASH_CTL;
	volatile uint32_t ROM_CTL;
};

struct PsocSpcifType_4000 {
	volatile uint32_t GEOMETRY;
	volatile uint32_t ADRCTRL;				/*PRIVILEDGED*/
	volatile uint32_t FLASH_OP_TIMER;		/*PRIVILEDGED*/
	volatile uint32_t FLASH_LL_CTRL;		/*PRIVILEDGED*/
	volatile uint32_t WRITE_LATCH_VAL;		/*PRIVILEDGED*/
	volatile uint32_t NVL_CTRL;				/*PRIVILEDGED*/
	volatile uint32_t NVL_RD_DATA;			/*PRIVILEDGED*/
	volatile uint32_t NVL_WR_DATA;			/*PRIVILEDGED*/
	uint32_t reserved1[2];
	volatile uint32_t NONBLOCKING_STATUS;	/*PRIVILEDGED*/
	uint32_t reserved2[1];
	volatile uint32_t FLASH_VIEWCTL;		/*PRIVILEDGED*/
	uint32_t reserved3[495];
	volatile uint32_t INTR;
	volatile uint32_t INTR_SET;
	volatile uint32_t INTR_MASK;
	volatile uint32_t INTR_MASKED;
	uint32_t reserved4[15808];
	struct {
		volatile uint32_t TRIM_M0_DAC0;			/*PRIVILEDGED*/
		volatile uint32_t ANALOG_READ_THRESH;	/*PRIVILEDGED*/	//(name suffix is to avoid shared code from 4100/4200 using this as it has different meaning)
		volatile uint32_t DAC_SETTINGS;			/*PRIVILEDGED*/
		volatile uint32_t BG_VPOS_TRIM;			/*PRIVILEDGED*/	//trim for VPOS bangap reference
	} PerMacroTrim[128];
};

struct PsocSpcifType_4100_4200 {
	volatile uint32_t GEOMETRY;
	volatile uint32_t ADRCTRL;				/*PRIVILEDGED*/
	volatile uint32_t FLASH_OP_TIMER;		/*PRIVILEDGED*/
	volatile uint32_t FLASH_LL_CTRL;		/*PRIVILEDGED*/
	volatile uint32_t WRITE_LATCH_VAL;		/*PRIVILEDGED*/
	volatile uint32_t NVL_CTRL;				/*PRIVILEDGED*/
	volatile uint32_t NVL_RD_DATA;			/*PRIVILEDGED*/
	volatile uint32_t NVL_WR_DATA;			/*PRIVILEDGED*/
	volatile uint32_t FLASH_VIEWCTL;		/*PRIVILEDGED*/
	uint32_t reserved1;
	volatile uint32_t NONBLOCKING_STATUS;	/*PRIVILEDGED*/
	volatile uint32_t DAC_SETTINGS;			/*PRIVILEDGED*/
	uint32_t reserved2[16308];
	struct {
		volatile uint32_t unknown_ff00_41_42;	/*PRIVILEDGED*/	//(name suffix is to avoid shared code from 4000 using this as it has different meaning)
		volatile uint32_t unknown_ff04_41_42;	/*PRIVILEDGED*/	//(name suffix is to avoid shared code from 4000 using this as it has different meaning)
		uint32_t reserved3[2];					//unverified that in these chips this struct is also 16 bytes
	} PerMacroTrim[128];
};

struct PsocSrsslt_4000 {
	volatile uint32_t PWR_CONTROL;
	volatile uint32_t PWR_KEY_DELAY;
	uint32_t reserved1;
	volatile uint32_t PWR_DDFT_SELECT;
	uint32_t reserved2;
	volatile uint32_t TST_MODE;
	uint32_t reserved3[4];
	volatile uint32_t CLK_SELECT;
	volatile uint32_t CLK_ILO_CONFIG;
	volatile uint32_t CLK_IMO_CONFIG;
	volatile uint32_t PWR_DFT_SELECT;
	volatile uint32_t WDT_DISABLE_KEY;
	volatile uint32_t WDT_COUNTER;
	volatile uint32_t WDT_MATCH;
	volatile uint32_t SRSS_INTR;
	volatile uint32_t SRSS_INTR_SET;
	volatile uint32_t SRSS_INTR_MASK;
	uint32_t reserved4;
	volatile uint32_t RES_CAUSE;
	uint32_t reserved5[940];
	volatile uint32_t CLK_IMO_SELECT;
	volatile uint32_t CLK_IMO_TRIM1;
	volatile uint32_t CLK_IMO_TRIM2;
	volatile uint32_t PWR_PWRSYS_TRIM1;
	volatile uint32_t CLK_IMO_TRIM3;
};

struct PsocSrss_4100_4200 {
	volatile uint32_t PWR_CONTROL;
	volatile uint32_t PWR_INTR;
	volatile uint32_t PWR_INTR_MASK;
	volatile uint32_t PWR_KEY_DELAY;
	volatile uint32_t PWR_PWRSYS_CONFIG;
	volatile uint32_t PWR_BG_CONFIG;
	volatile uint32_t PWR_VMON_CONFIG;
	volatile uint32_t PWR_DFT_SELECT;
	volatile uint32_t PWR_DDFT_SELECT;
	volatile uint32_t PWR_DFT_KEY;
	volatile uint32_t PWR_BOD_KEY;
	volatile uint32_t PWR_STOP;
	uint32_t reserved1[52];
	volatile uint32_t CLK_SELECT;
	volatile uint32_t CLK_ILO_CONFIG;
	volatile uint32_t CLK_IMO_CONFIG;
	volatile uint32_t CLK_IMO_SPREAD;
	volatile uint32_t CLK_DFT_SELECT;
	uint32_t reserved2[59];
	volatile uint32_t WDT_CTRLOW;
	volatile uint32_t WDT_CTRHIGH;
	volatile uint32_t WDT_MATCH;
	volatile uint32_t WDT_CONFIG;
	volatile uint32_t WDT_CONTROL;
	uint32_t reserved3[59];
	volatile uint32_t RES_CAUSE;
	uint32_t reserved4[16127];
	volatile uint32_t PWR_PWRSYS_TRIM1;
	volatile uint32_t PWR_PWRSYS_TRIM2;
	volatile uint32_t PWR_PWRSYS_TRIM3;
	volatile uint32_t PWR_PWRSYS_TRIM4;
	volatile uint32_t PWR_BG_TRIM1;
	volatile uint32_t PWR_BG_TRIM2;
	volatile uint32_t PWR_BG_TRIM3;
	volatile uint32_t PWR_BG_TRIM4;
	volatile uint32_t PWR_BG_TRIM5;
	volatile uint32_t CLK_ILO_TRIM;
	volatile uint32_t CLK_IMO_TRIM1;
	volatile uint32_t CLK_IMO_TRIM2;
	volatile uint32_t CLK_IMO_TRIM3;
	volatile uint32_t CLK_IMO_TRIM4;
	volatile uint32_t PWR_RSVD_TRIM;
};

struct PsocTest_4100_4200 {
	volatile uint32_t DEBUGGER_LIMITS;	/*PRIVILEDGED*/		//0x04 is set if debugging is connected. set 0x02 to disable debugger from ram & flash access. set 1 to disable debugger entirely
	uint32_t reserved1[4];
	volatile uint32_t MODE;
};

struct PsocScb {
	volatile uint32_t CTRL;
	volatile uint32_t STATUS;
	uint32_t reserved1[6];
	volatile uint32_t SPI_CTRL;
	volatile uint32_t SPI_STATUS;
	uint32_t reserved2[6];
	volatile uint32_t UART_CTRL;
	volatile uint32_t UART_TX_CTRL;
	volatile uint32_t UART_RX_CTRL;
	volatile uint32_t UART_RX_STATUS;
	uint32_t reserved3[4];
	volatile uint32_t I2C_CTRL;
	volatile uint32_t I2C_STATUS;
	volatile uint32_t I2C_M_CMD;
	volatile uint32_t I2C_S_CMD;
	volatile uint32_t I2C_CFG;
	uint32_t reserved4[35];
	volatile uint32_t BIST_CONTROL;
	volatile uint32_t BIST_DATA;
	uint32_t reserved5[62];
	volatile uint32_t TX_CTRL;
	volatile uint32_t TX_FIFO_CTRL;
	volatile uint32_t TX_FIFO_STATUS;
	uint32_t reserved6[13];
	volatile uint32_t TX_FIFO_WR;
	uint32_t reserved7[47];
	volatile uint32_t RX_CTRL;
	volatile uint32_t RX_FIFO_CTRL;
	volatile uint32_t RX_FIFO_STATUS;
	volatile uint32_t RX_MATCH;
	uint32_t reserved8[11];
	volatile uint32_t RX_FIFO_RD;
	volatile uint32_t RX_FIFO_RD_SILENT;
	uint32_t reserved9[46];
	volatile uint32_t EZ_DATA[32];
	uint32_t reserved10[608];
	volatile uint32_t INTR_CAUSE;
	uint32_t reserved11[31];
	volatile uint32_t INTR_I2C_EC;
	uint32_t reserved12[1];
	volatile uint32_t INTR_I2C_EC_MASK;
	volatile uint32_t INTR_I2C_EC_MASKED;
	uint32_t reserved13[12];
	volatile uint32_t INTR_SPI_EC;
	uint32_t reserved14[1];
	volatile uint32_t INTR_SPI_EC_MASK;
	volatile uint32_t INTR_SPI_EC_MASKED;
	uint32_t reserved15[12];
	volatile uint32_t INTR_M;
	volatile uint32_t INTR_M_SET;
	volatile uint32_t INTR_M_MASK;
	volatile uint32_t INTR_M_MASKED;
	uint32_t reserved16[12];
	volatile uint32_t INTR_S;
	volatile uint32_t INTR_S_SET;
	volatile uint32_t INTR_S_MASK;
	volatile uint32_t INTR_S_MASKED;
	uint32_t reserved17[12];
	volatile uint32_t INTR_TX;
	volatile uint32_t INTR_TX_SET;
	volatile uint32_t INTR_TX_MASK;
	volatile uint32_t INTR_TX_MASKED;
	uint32_t reserved18[12];
	volatile uint32_t INTR_RX;
	volatile uint32_t INTR_RX_SET;
	volatile uint32_t INTR_RX_MASK;
	volatile uint32_t INTR_RX_MASKED;
};

struct PsocDmacChAddr {
	volatile uint32_t SRC;
	volatile uint32_t DST;
	volatile uint32_t CTL;
	volatile uint32_t STATUS;
};

struct PsocDmacCh {
	struct PsocDmacChAddr ping;
	struct PsocDmacChAddr pong;
};

struct PsocDmac {
	volatile uint32_t CTL;
	uint32_t reserved1[3];
	volatile uint32_t STATUS;
	volatile uint32_t STATUS_SRC_ADDR;
	volatile uint32_t STATUS_DST_ADDR;
	volatile uint32_t STATUS_CH_ACT;
	uint32_t reserved2[24];
	volatile uint32_t CH_CTL[8];
	uint32_t reserved3[468];
	volatile uint32_t INTR;
	volatile uint32_t INTR_SET;
	volatile uint32_t INTR_MASK;
	volatile uint32_t INTR_MASKED;
	struct PsocDmacCh ch[8];
};

struct PsocPeriTrGroup{
	volatile uint32_t TR_OUT_CTL[8];
};

struct PsocPeriType {
	volatile uint32_t DIV_CMD;
	uint32_t reserved1[63];
	volatile uint32_t PCLK_CTL[24];
	uint32_t reserved2[104];
	volatile uint32_t DIV_16_CTL[16];
	uint32_t reserved3[48];
	volatile uint32_t DIV_16_5_CTL[4];
	uint32_t reserved4[124];
	volatile uint32_t PERI_TR_CTL;
};

struct PsocTcpwm {
	volatile uint32_t CTRL;
	uint32_t reserved1[1];
	volatile uint32_t CMD;
	volatile uint32_t INTR_CAUSE;
	uint32_t reserved2[60];
	struct PsocTcpwmUnit {
		volatile uint32_t CTRL;
		volatile uint32_t STATUS;
		volatile uint32_t COUNTER;
		volatile uint32_t CC;
		volatile uint32_t CC_BUFF;
		volatile uint32_t PERIOD;
		volatile uint32_t PERIOD_BUFF;
		uint32_t reserved1[1];
		volatile uint32_t TR_CTRL[3];
		uint32_t reserved2[1];
		volatile uint32_t INTR;
		volatile uint32_t INTR_SET;
		volatile uint32_t INTR_MASK;
		volatile uint32_t INTR_MASKED;
	} cnt[8];
};

#ifdef _CPU_PSOC4000S
	#define __MPU_PRESENT	1
	#define __VTOR_PRESENT	1
	#include "core_cm0plus.h"
#else
	#include "core_cm0.h"
#endif
#include "core_cmInstr.h"

#define CPUSS_BASE_NEW	0x40100000ul
#define CPUSS_BASE_OLD	0x40000000ul
#define SPCIF_BASE_NEW	0x40110000ul
#define SPCIF_BASE_OLD	0x400e0000ul

#if defined(_CPU_PSOC4000) || defined(_CPU_PSOC4000S) || defined(_CPU_PSOC4200M) || defined(_CPU_PSOC4100M) || defined(_CPU_PSOC4_CYBL10x6x)
	#define CPUSS_BASE		CPUSS_BASE_NEW
	#define SPCIF_BASE		SPCIF_BASE_NEW
#elif defined(_CPU_PSOC4100) || CPUSS_BASE_OLD(_CPU_PSOC4200)
	#define CPUSS_BASE		CPUSS_BASE_OLD
	#define SPCIF_BASE		SPCIF_BASE_OLD
#else
	#error "PSoC CPU type not defined. Please #define _CPU_PSOC4*"
#endif

#if defined(_CPU_PSOC4000) || defined(_CPU_PSOC4000S) || defined(_CPU_PSOC4200M) || defined(_CPU_PSOC4100M) || defined(_CPU_PSOC4_CYBL10x6x)
	#define PsocSpcifType	PsocSpcifType_4000
	#define EXTRA_FLASH_MACROS_HAVE_SFLASH
#endif

#if defined(_CPU_PSOC4100) || defined(_CPU_PSOC4200)
	#define PsocSpcifType	PsocSpcifType_4100_4200
#endif

#if defined(_CPU_PSOC4000) || defined(_CPU_PSOC4000S)
	#define SRSSLT			((struct PsocSrsslt_4000*)0x40030000ul)
#endif

#if defined(_CPU_PSOC4100) || defined(_CPU_PSOC4200) || defined(_CPU_PSOC4200M) || defined(_CPU_PSOC4100M) || defined(_CPU_PSOC4_CYBL10x6x)
	#define SRSS			((struct PsocSrss_4100_4200*)0x400B0000ul)
	#define TST				((struct PsocTest_4100_4200*)0x40030000ul)
#endif



#define HSIOM_PRT		((struct PsocHsiomPrtType*)		0x40020000ul)
#define GPIO_PRT		((struct PsocGpioPrtType*)		0x40040000ul)
#define CPUSS			((struct PsocCpussType*)		CPUSS_BASE)
#define SPCIF			((struct PsocSpcifType*)		SPCIF_BASE)
#define SCB0			((struct PsocScb*)				0x40060000ul)
#define SCB1			((struct PsocScb*)				0x40070000ul)
#define DMAC			((struct PsocDmac*)				0x40101000ul)
#define PERI_TR_GROUP	((struct PsocPeriTrGroup*)		0x40012000ul)
#define PERI			((struct PsocPeriType*)			0x40010000ul)
#define TCPWM			((struct PsocTcpwm*)			0x40200000ul)


#define SFLASH_START		0x0FFFF000ul

#ifdef __cplusplus
}
#endif

#endif
